#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>

#define MAXCHAR 72

const char *szCommand = "";        /* Name dieses Programms, wird erst gesetzt */

static int pPipe[2];
char    buffer[80];  
int     nbytes;
pid_t child_pid, wpid;
int status;



void FreeRessources(void){
        if (close(pPipe[0]) == -1)
                printf("Unimportant Error while closing pipe");
        if (close(pPipe[1]) == -1)
                printf("Unimportant Error while closing pipe");
        printf("\nRessources free!\n");                 
}


void BailOut(char *error){
    (void) fprintf(stderr, "%s: %s: %s\n", szCommand, error, strerror(errno));
    FreeRessources();
    exit(EXIT_FAILURE);
}



void AllocateRessources(void) {
        if (pipe(pPipe) == -1) {
                BailOut("Cannot create pipe!");
        }
}


void sig_handler(int signal){
    (void) fprintf(stderr, "Caught signal %d\n", signal);
    FreeRessources();
    exit(EXIT_SUCCESS);
}

void ChildProcess(){
        char line[80];
        int retv;
        if (close(pPipe[0]) == -1)
                BailOut("Could not close pPipe[0]");
     	printf("tlsh# ");
		for (; fgets(line,sizeof(line),stdin) != NULL; printf("tlsh# "))
		{
/*
        while(fgets(line,sizeof(line),stdin) != NULL)
                fgets(line,sizeof(line),stdin);
                fprintf(stdout, line);
*/
				line[strlen(line)-1] = '\0';
                retv = write(pPipe[1],line,strlen(line));
                if (retv == -1) {
                        BailOut("Error while writing to pipe");
                }
				sleep(5);
        }
/*
        retv = write(pPipe[1],"EOF",3);
        if (retv == -1) {
                BailOut("Error while writing to pipe");
        }
*/
        if (close(pPipe[1]) == -1) 
                BailOut("Could not close pPipe[1]");
        return;
}

void ProcessOutput(FILE *fp){
        char line[80];
        while(fgets(line, sizeof(line),fp) != NULL) {
                printf(line);
        }
        
}

void FatherProcess() {
        FILE *fp;
        if (close(pPipe[1]) == -1) 
                BailOut("Could not close pPipe[1]");
        for (;;) {
                nbytes = read(pPipe[0], buffer, sizeof(buffer));
                if (nbytes== 0) {
                        printf("Exiting..\n");
						exit(EXIT_SUCCESS);
                }       
                if (nbytes == -1) {
                        BailOut("Error while reading from pipe");
                }
                printf("Received String: %s\n", buffer);
/*
                if ( strcmp(buffer, "EOF") == 0) {
                        printf("Exiting..");
                        FreeRessources();
                        exit(EXIT_SUCCESS);
                }
*/
                if(( fp = popen(buffer,"r")) != NULL) {
                        printf("Programm ausgefhrt\n");
                        ProcessOutput(fp);
                        pclose(fp);
                }
        }       
}

int main(int argc, char **argv)
{
    /* Signal-Handler installieren */
    (void) signal(SIGINT,  sig_handler);
    (void) signal(SIGTERM, sig_handler);
    (void) signal(SIGQUIT, sig_handler);
    (void) signal(SIGPIPE, sig_handler);
    szCommand = argv[0];

    AllocateRessources();
    fprintf(stdout, "Forking..\n");       
    child_pid = fork();
    switch(child_pid){
        case -1:
            BailOut("fork() failed");
            break;

        case 0:
            ChildProcess();
            break;
        default:
            FatherProcess();
                
            while((wpid = wait(&status)) != child_pid) {
                if (wpid != -1) {
                        continue;
                }
                if (errno == EINTR) {
                        continue;
                }
                BailOut("Error waiting for child process");
            }
            break;
    }

    exit(EXIT_SUCCESS);
}
